package googlehack;

import java.util.Iterator;
import java.net.URL;
import java.net.URI;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import org.json.*;

public class GoogleHack {
    public GoogleHack() {
    }

    public SearchResponse search(String searchString) throws SearchException {
		return search(searchString, false);
	}
	
    public SearchResponse search(String searchString, boolean debug) throws SearchException {
		if (searchString == null) {
	    	System.err.println("Null search string, nothing to search for");
	    	return null;
		}

		if (searchString.equals("")) {
	    	System.err.println("Empty search string, nothing to search for");
	    	return null;
		}

		URL url = null;
		
		String keyString = "";
		String googleKey = System.getenv("GOOGLE_KEY");
		
		if (googleKey == null || googleKey.equals("")) {
			googleKey = "";
		} else {
			googleKey = "key=" + googleKey + "&";
			if (debug) {
				System.out.println("Using google key = " + googleKey);
			}
		}

		// Create properly quoted uri
		try {
	    	URI uri = new URI("http", 									// scheme
	                      	"ajax.googleapis.com", 						// authority
	                      	"/ajax/services/search/web",				// path
	                      	"v=1.0&" + googleKey + "q=" + searchString,	// query
	                      	null);										// fragment
	    	url = uri.toURL();
		} catch (URISyntaxException ex) {
	    	throw new SearchException("Error creating URI from '" + searchString + 
				"': " + ex.toString());
		} catch (IllegalArgumentException ex) {
	    	throw new SearchException("Error creating absolute URL from '" +
				searchString + "': " + ex.toString());
		} catch (MalformedURLException ex) {
	    	throw new SearchException("Error creating URL from '" + searchString + 
				"': " + ex.toString());
		}

		// TODO
		// 1. Add support for API key - DONE!
		// 2. Add userip parameter to query
		// 3. Add Referer property

		if (debug) {
			System.out.println("URL: " + url);
		}
		
		StringBuffer response = new StringBuffer();
		BufferedReader in = null;

		try {
	    	in = new BufferedReader(new InputStreamReader(url.openStream()));
	    	String line;
	    	while ((line = in.readLine()) != null) {
	       		response.append(line);	
	    	}
		} catch (IOException ex) {
	    	throw new SearchException("Error reading from Google: " + ex.toString());
		} finally {
	    	try {
				if (in != null) {
		    		in.close();
				}
	    	} catch (IOException ex2) {
				// Silently consume the exception
	    	}
		}
		
		if (debug) {
			System.out.println("Raw response from server: " + response.toString());
		}

		JSONObject json = null;

		try {
	    	json = new JSONObject(response.toString());
		} catch (JSONException ex) {
	    	throw new SearchException("Error in JSON data from Google: " + ex + "\n" +
	    		"Original data: " + response.toString());
		}

		SearchResponse resp = new SearchResponse(json);

		if (debug) {
			System.out.println("Response: " + resp);
		}
		
		return resp;
    }

   	public static void main(String[] argv) {
		if (argv.length != 1) {
				System.err.println("Usage: GoogleHack <searchterm>");
				System.err.println("If using a multi-word search term, enclose the phrase in quotes");
				System.exit(1);
		}
		
		GoogleHack gh = new GoogleHack();
		System.out.println("Searching for " + argv[0]);
		
		try {
			SearchResponse resp = gh.search(argv[0]);
		
			if (resp != null) {
    			System.out.println("Received: " + resp);
			} else {
    			System.out.println("Null reply");
			}
		} catch (SearchException ex) {
			System.err.println("Error during search: " + ex.toString());
			ex.printStackTrace();
		}
    }
}
