package googlehack;

import org.json.*;

public class SearchResponse {
    // consts for some keys (not all are currently used)
    private static final String RESPONSE_DATA = "responseData";
    private static final String RESPONSE_STATUS = "responseStatus";
    private static final String RESPONSE_DETAILS = "responseDetails";
    private static final String RESULTS = "results";
    private static final String CURSOR = "cursor";
    private static final String PAGES = "pages";
    private static final String START = "start";
    private static final String LABEL = "label";
    private static final String ESTIMATED_RESULT_COUNT = "estimatedResultCount";
    private static final String CURRENT_PAGE_INDEX = "currentPageIndex";
    private static final String MORE_RESULTS_URL = "moreResultsUrl";
    private static final String GSEARCH_RESULT_CLASS = "GsearchResultClass";
    private static final String UNESCAPED_URL = "unescapedUrl";
    private static final String URL = "url";
    private static final String VISIBLE_URL = "visibleUrl";
    private static final String CACHE_URL = "cacheUrl";
    private static final String TITLE = "title";
    private static final String TITLE_NO_FORMATTING = "titleNoFormatting";
    private static final String CONTENT = "content";

    private int responseStatus;		// 200 for success
    private String responseDetails; 	// diagnostic info if status != 200
    private SearchResult[] results;
    // omitting cursor stuff for now
    // private int estimatedPageCount;
    // private int currentPageIndex;
    // private URL moreResultsURL;

    public SearchResponse(JSONObject json) throws SearchException {
		JSONObject responseData = null;

		////////////////////////////////////////////////////////////////
		// responseData - required, toplevel element
		try {
	    	if (!json.has(RESPONSE_DATA)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			RESPONSE_DATA + " key.");
	    	}

	    	responseData = json.getJSONObject(RESPONSE_DATA);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				RESPONSE_DATA + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// responseDetails - optional, diagnostic info if status != 200
		try {
	    	if (!json.has(RESPONSE_DETAILS)) {
				this.responseDetails = null;
	    	} else {
				this.responseDetails = json.getString(RESPONSE_DETAILS);	
	    	}
		} catch (JSONException ex) {
			ex.printStackTrace();
			throw new SearchException("JSON error getting " + 
				RESPONSE_DETAILS + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// responseStatus - required, 200 if success
		try {
	    	if (!json.has(RESPONSE_STATUS)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			RESPONSE_STATUS + " key.");
	    	} else {
				this.responseStatus = json.getInt(RESPONSE_STATUS);	
	    	}
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				RESPONSE_DETAILS + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// results - array of search results
		try {
	    	if (!responseData.has(RESULTS)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			RESULTS + " key.");
	    	} else {
				JSONArray arr = responseData.getJSONArray(RESULTS);
				this.results = new SearchResult[arr.length()];
				for (int i=0, len=arr.length() ; i<len ; i++) {
		    		JSONObject obj = null;
		    		try {
		        		obj = arr.getJSONObject(i);	
		        		this.results[i] = new SearchResult(obj);
		    		} catch (JSONException ex) {
						throw new SearchException("JSON error getting result " +
							i + ": " + ex.toString());
		    		}
				}
	    	}
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				RESULTS + ": " + ex.toString());
		}
    }

    public int getResponseStatus() {
		return responseStatus;
    }

    public String getResponseDetails() {
		return responseDetails;
    }

    public SearchResult[] getResults() {
		return results;
    }

    public String toString() {
		StringBuffer buf = new StringBuffer();

		if (getResponseStatus() == 200) {
	    	buf.append("Response Status = 200 (Success)\n");
		} else {
	    	buf.append("Response Status = " + getResponseStatus()
				+ " (Response Details = " + getResponseDetails() 
				+ ")\n");
		}

		SearchResult[] results = getResults();

		if (results.length == 0) {
	    	buf.append("0 results found\n");
		} else {
	    	buf.append(results.length + " results follow\n");

	    	for (int i=0, len=results.length ; i<len ; i++) {
	        	buf.append("Result " + i + ":" + "\n");
				buf.append(results[i].toString() + "\n");
	    	}
		}
		
		return buf.toString();
    }
}

