package googlehack;

import java.net.URL;
import java.net.MalformedURLException;
import org.json.*;

public class SearchResult {
    private static final String GSEARCH_RESULT_CLASS = "GsearchResultClass";
    private static final String UNESCAPED_URL = "unescapedUrl";
    private static final String URL = "url";
    private static final String VISIBLE_URL = "visibleUrl";
    private static final String CACHE_URL = "cacheUrl";
    private static final String TITLE = "title";
    private static final String TITLE_NO_FORMATTING = "titleNoFormatting";
    private static final String CONTENT = "content";
	
    private String gSearchResultClass;
    private String unescapedURL;
    private URL url;
    private String visibleURL;
    private URL cacheURL;
    private String title;
    private String titleNoFormatting;
    private String content;

    public SearchResult(JSONObject json) throws SearchException {
		////////////////////////////////////////////////////////////////
		// GsearchResultClass -- GwebResult, local, video, etc...
		try {
	    	if (!json.has(GSEARCH_RESULT_CLASS)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			GSEARCH_RESULT_CLASS + " key.");
	    	}

	    	gSearchResultClass = json.getString(GSEARCH_RESULT_CLASS);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				GSEARCH_RESULT_CLASS + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// unescapedUrl
		try {
	    	if (!json.has(UNESCAPED_URL)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			UNESCAPED_URL + " key.");
	    	}

	    	unescapedURL = json.getString(UNESCAPED_URL);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				UNESCAPED_URL + ": " + ex.toString());
		}


		////////////////////////////////////////////////////////////////
		// url
		try {
	    	if (!json.has(URL)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			URL + " key.");
	    	}

	    	url = new URL(json.getString(URL));
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				URL + ": " + ex.toString());
		} catch(MalformedURLException ex) {
			throw new SearchException("Malformed URL: " + 
				ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// visibleUrl
		try {
	    	if (!json.has(VISIBLE_URL)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			VISIBLE_URL + " key.");
	    	}

	    	visibleURL = json.getString(VISIBLE_URL);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				VISIBLE_URL + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// cacheUrl
		try {
	    	if (!json.has(CACHE_URL)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			CACHE_URL + " key.");
	    	}

	    	cacheURL = new URL(json.getString(CACHE_URL));
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				CACHE_URL + ": " + ex.toString());
		} catch (MalformedURLException ex) {
			throw new SearchException("Malformed URL: " + 
				ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// title
		try {
	    	if (!json.has(TITLE)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			TITLE + " key.");
	    	}

	    	title = json.getString(TITLE);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				TITLE + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// titleNoFormatting
		try {
	    	if (!json.has(TITLE_NO_FORMATTING)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			TITLE_NO_FORMATTING + " key.");
	    	}

	    	titleNoFormatting = json.getString(TITLE_NO_FORMATTING);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				TITLE_NO_FORMATTING + ": " + ex.toString());
		}

		////////////////////////////////////////////////////////////////
		// content
		try {
	    	if (!json.has(CONTENT)) {
	        	throw new SearchException("Google response doesn't contain " +
	    			CONTENT + " key.");
	    	}

	    	content = json.getString(CONTENT);
		} catch (JSONException ex) {
			throw new SearchException("JSON error getting " + 
				CONTENT + ": " + ex.toString());
		}
	}

    public String getGSearchResultClass() {
		return gSearchResultClass;
    }

    public String getUnescapedURL() {
		return unescapedURL;
    }

    public URL getURL() {
		return url;
    }

    public String getVisibleURL() {
		return visibleURL;
    }

    public URL getCacheURL() {
		return cacheURL;
    }

    public String getTitle() {
		return title;
    }

    public String getTitleNoFormatting() {
		return titleNoFormatting;
    }

    public String getContent() {
		return content;
    }

    public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("\tURL = " + getURL() + "\n");
		buf.append("\tTitle = " + getTitle() + "\n");
		buf.append("\tTitle (no formatting) = " + getTitleNoFormatting() + "\n");
		buf.append("\tContent = " + getContent());
		return buf.toString();
	}
}

